@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -31,6 +31,9 @@ USA.
 
 ;;;; Architecture Parameters
 
+(define (compiler-features) '(target-arch=c))
+(define (target-fasl-format) (error "I should not be fasdumping crap!"))
+
 (define use-pre/post-increment? true)
 (define endianness 'DONT-KNOW)
 (define scheme-object-width "OBJECT_LENGTH")
@@ -302,4 +305,11 @@ USA.
 
 (define compiler:primitives-with-no-open-coding
   '(DIVIDE-FIXNUM GCD-FIXNUM &/
-    VECTOR-CONS STRING-ALLOCATE FLOATING-VECTOR-CONS))
\ No newline at end of file
+    VECTOR-CONS STRING-ALLOCATE FLOATING-VECTOR-CONS
+    FLONUM-FMA
+    FLONUM-IS-LESS? FLONUM-IS-LESS-OR-EQUAL?
+    FLONUM-IS-GREATER? FLONUM-IS-GREATER-OR-EQUAL?
+    FLONUM-IS-LESS-OR-GREATER? FLONUM-IS-UNORDERED?
+    FLONUM-IS-NORMAL? FLONUM-IS-FINITE? FLONUM-IS-INFINITE? FLONUM-IS-NAN?
+    FLONUM-IS-ZERO? FLONUM-IS-EQUAL?
+    FLONUM-IS-NEGATIVE? FLONUM-COPYSIGN))
\ No newline at end of file
