@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -30,7 +30,7 @@ USA.
 	 (integrate-external "object"))
 
 (define (read-package-model filename os-type)
-  (let ((model-pathname (merge-pathnames filename)))
+  (let ((model-pathname (cref/source-pathname filename)))
     (receive (packages extensions loads globals)
 	(sort-descriptions (read-and-parse-model model-pathname os-type))
       (descriptions->pmodel
@@ -65,9 +65,16 @@ USA.
 			    (system-library-pathname pkd)))))
 	   (and (not (condition? pathname))
 		pathname))
-	 (let ((pathname (merge-pathnames pkd model-pathname)))
-	   (and (file-exists? pathname)
-		pathname)))
+	 (or (let* ((object-model-pathname
+		     (merge-pathnames
+		      (enough-pathname model-pathname cref/source-root)
+		      cref/object-root))
+		    (pathname (merge-pathnames pkd object-model-pathname)))
+	       (and (file-exists? pathname)
+		    pathname))
+	     (let ((pathname (merge-pathnames pkd model-pathname)))
+	       (and (file-exists? pathname)
+		    pathname))))
      (begin
        (warn "Could not find global definitions:" pkd)
        #f))))
@@ -146,9 +153,9 @@ USA.
 
 (define (cache-file-analyses! pmodel os-type)
   (let ((pathname
-	 (pathname-new-type (package-set-pathname (pmodel/pathname pmodel)
-						  os-type)
-			    "fre"))
+	 (pathname-new-type
+	  (package-set-pathname (pmodel/object-pathname pmodel) os-type)
+	  "fre"))
 	(changes? (list #f)))
     (let ((result
 	   (let ((caches
@@ -174,8 +181,9 @@ USA.
 (define (cache-file-analysis! pmodel caches pathname changes?)
   (let ((cache (analysis-cache/lookup caches pathname))
 	(full-pathname
-	 (merge-pathnames (pathname-new-type pathname "bin")
-			  (pmodel/pathname pmodel))))
+	 (merge-pathnames
+	  (pathname-new-type pathname (if sf/cross-compiling? "nib" "bin"))
+	  (pmodel/object-pathname pmodel))))
     (let ((time (file-modification-time full-pathname)))
       (if (not time)
 	  (error "unable to open file" full-pathname))
@@ -372,27 +380,12 @@ USA.
 			(cond ((and (pair? (cdr option))
 				    (eq? 'deprecated (cadr option)))
 			       (parse-import/export (cddr option) #t))
-			      ;; 9.2 compatibility
-			      ((and (pair? (cdr option))
-				    (pair? (cddr option))
-				    (symbol? (caddr option))
-				    (string-prefix-ci?
-				     "deprecated:"
-				     (symbol->string (caddr option))))
-			       (parse-import/export (cons (cadr option)
-							  (cdddr option))
-						    #t))
 			      (else
 			       (parse-import/export (cdr option) #f)))))
 		   (set-package-description/exports!
 		    package
 		    (append! (package-description/exports package)
 			     (list export)))))
-		((export-deprecated)
-		 (set-package-description/exports!
-		  package
-		  (append! (package-description/exports package)
-			   (list (parse-import/export (cdr option) #t)))))
 		((import)
 		 (set-package-description/imports!
 		  package
